--[[
Plugin that handles filtering of chat and raid warnings
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Filter")
if not plugin then return end

----------------------------------
-- Locals 

local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")

local db
local inCombat = nil
local party
local whisperSpamm 

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Filter", { profile = {} })
	db = self.db.profile
end

function plugin:PluginEnable()
	self:RegisterMessage("RW_BossEngaged")
	self:RegisterMessage("RW_BossEnded")
	party = RW:GetPlugin("Party")
	whisperSpamm = {}
end

function plugin:PluginDisable()
end

function plugin:RW_BossEngaged(message, boss)
	inCombat = boss
	self:RegisterEvent("CHAT_MSG_WHISPER")
end

function plugin:RW_BossEnded(message, boss)
	inCombat = nil
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	wipe(whisperSpamm)
end

local function removeSpamm(name)
	whisperSpamm[name] = nil
end


function plugin:CHAT_MSG_WHISPER(event, msg, sender)
	if inCombat and msg == "status" then
		SendChatMessage(L.RW_CHAT_PREFIX ..L.RW_CHAT_RESPONSE:format(
			inCombat:GetName(), 
			inCombat:GetBossHealth() * 100,
			RW:TimeToStringShort(GetTime() - inCombat.engageTime),
			party:GetNumAlive(), 
			party:GetPartySize()), "WHISPER", nil, sender)
			
	elseif inCombat and not whisperSpamm[sender] and not party.group[sender] then
		whisperSpamm[sender] = true
		SendChatMessage(L.RW_CHAT_PREFIX ..L.RW_CHAT_RESPONSE:format(
			inCombat:GetName(), 
			inCombat:GetBossHealth() * 100, 
			RW:TimeToStringShort(GetTime() - inCombat.engageTime),
			party:GetNumAlive(), 
			party:GetPartySize()), "WHISPER", nil, sender)
		inCombat:Schedule("removeSpamm", 10, removeSpamm, sender)
	end
end

local function filterWhisperOut(self, event, ...)
	local msg = ...
	if not msg then return end
	return msg:sub(1, L.RW_CHAT_PREFIX:len()) == L.RW_CHAT_PREFIX, ...
end
	
local function filterWhisperIn(self, event, ...)
	local msg = ...
	if not msg then return false, ... end
	return inCombat and msg == "status", ...
end

local function filterRaidWarning(self, event, ...)
	local msg = ...
	if not msg then return false, ... end
	return msg:match("%*%*%* .* %*%*%*"), ...
end
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING", filterRaidWarning)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", filterWhisperIn)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filterWhisperOut)

local old = RaidWarningFrame:GetScript("OnEvent")
RaidWarningFrame:SetScript("OnEvent", function(self, event, msg, ...)
	if msg:match("%*%*%* .* %*%*%*") then
		return
	end
	return old(self, event, msg, ...)
end)

-------------------------------
-- Options
-- function plugin:GetOptions()
	-- self.category = "Misc Plugins"
	-- local options = {
		-- type = "group",
		-- name = "Chat filter",
		-- icon = [[Interface\CHATFRAME\UI-ChatIcon-Chat-Down]],
		-- args = {
		-- }
	-- }
	
	-- return options
-- end