--[[
Plugin for range check
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("RangeCheck")

----------------------------------
-- Locals
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")
local Party

local db


local defaults = {
	profile = {
		PluginEnabled = true,
		Width = 180,
		range = 15,
		maxNames = 5,
		GrowUp = true,
	}
}

for i, setting in pairs(RW.defaults.profile) do
	defaults.profile[i] = setting
end

local inset = defaults.profile.BorderSize > 2 and defaults.profile.Insets or 0
local backdrop = {
	tile = false,
	insets = { left = inset, right = inset, top = inset, bottom = inset },
}

--------------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("RangeCheck", defaults)
	db = self.db.profile
end

function plugin:PluginEnable()
	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_DefaultsChanged")
	self:RegisterMessage("RW_ShowRangeCheck")
	Party = RW:GetPlugin("Party")
end

function plugin:PluginDisable()
end

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local s = self:GetEffectiveScale()
	db.x = self:GetLeft() * s
	db.y = self:GetBottom() * s
end

------------------------------------
-- Anchor

local function getAnchor()
	if plugin.anchor then return plugin.anchor end
	local frame = RW:CreateAnchor(plugin:GetName(), UIParent, "Range Check")
	frame:SetScript("OnDragStart", dragStart)
	frame:SetScript("OnDragStop", dragStop)
	frame:ClearAllPoints()
	local s = frame:GetEffectiveScale()
	if db.x then
		frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.x / s, db.y / s)
	else
		frame:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -400, 400)
	end
	frame:SetWidth(db.Width)
	plugin.anchor = frame
	return frame
end

local function onUpdate(self, elapsed)
	self.t = (self.t or 0) + elapsed
	if self.t > 0.5 then
		local numFound = 0
		local partyType = Party:GetPartyType()
		for i = 1, Party:GetPartySize() do
			local uID = partyType..i
			if (not UnitIsUnit("player", uID)) and (not UnitIsDeadOrGhost(uID) and plugin:CheckRange(uID)) then
				numFound = numFound + 1
				self.names[numFound]:SetText(UnitName(uID))
				local color = RAID_CLASS_COLORS[select(2, UnitClass(uID))]
				self.names[numFound]:SetTextColor(color.r, color.g, color.b)
				self.names[numFound]:Show()
			end
			if numFound >= plugin.db.profile.maxNames then break end
		end
		for i = numFound + 1, #self.names do
			self.names[i]:SetText()
			self.names[i]:Hide()
		end
		self:SetHeight(8 + self.rangeString:GetStringHeight() + 6 + numFound * (self.names[1]:GetStringHeight() + 6))
		self.t = 0
	end
end

local function getMainWindow()
	if plugin.mainWindow then return plugin.mainWindow end
	local anchor = getAnchor()
	
	local frame = CreateFrame("Frame", "RWRFrame", UIParent)
	frame:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT")
	frame:EnableMouse(true)
	frame:RegisterForDrag("LEFTBUTTON")
	frame:SetMovable(true)
	frame:SetHeight(30)
	frame:SetWidth(db.Width)
	
	local rangeString = frame:CreateFontString("RWR_TitleString", "ARTWORK")
	rangeString:SetPoint("TOPLEFT", frame, "TOPLEFT", 6, -6)
	frame.rangeString = rangeString
	
	--Close button
	local closebutton = CreateFrame("Button", "RWR_CloseButton", frame)
	closebutton:SetWidth(14)
	closebutton:SetHeight(14)
	closebutton:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -4, -6)
	
	local t = closebutton:CreateTexture()
	t:SetAllPoints(closebutton)
	t:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	t:SetTexCoord(0.2, 0.8, 0.2, 0.8)
	closebutton:SetNormalTexture(t)
	
	t = closebutton:CreateTexture(nil, "BACKGROUND")
	t:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	t:SetAllPoints(closebutton)
	t:SetTexCoord(0.2, 0.8, 0.2, 0.8)
	closebutton:SetPushedTexture(t)
	
	t = closebutton:CreateTexture()
	t:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	t:SetAllPoints(closebutton)
	t:SetBlendMode("ADD")
	t:SetTexCoord(0.2, 0.8, 0.2, 0.8)
	closebutton:SetHighlightTexture(t)
	
	local buttonUp = CreateFrame("Button", "RWR_UpButton", frame)
	buttonUp:SetWidth(16)
	buttonUp:SetHeight(16)
	buttonUp:SetPoint("TOPRIGHT", closebutton, "TOPLEFT", -4, 0)
	
	local t = buttonUp:CreateTexture()
	t:SetAllPoints(buttonUp)
	t:SetTexture("Interface\\Buttons\\UI-PlusButton-Up")
	buttonUp:SetNormalTexture(t)
	
	t = buttonUp:CreateTexture(nil, "BACKGROUND")
	t:SetTexture("Interface\\Buttons\\UI-PlusButton-Down")
	t:SetAllPoints(buttonUp)
	buttonUp:SetPushedTexture(t)
	
	t = buttonUp:CreateTexture()
	t:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight")
	t:SetAllPoints(buttonUp)
	t:SetBlendMode("ADD")
	buttonUp:SetHighlightTexture(t)
	
	local buttonDown = CreateFrame("Button", "RWR_UpButton", frame)
	buttonDown:SetWidth(16)
	buttonDown:SetHeight(16)
	buttonDown:SetPoint("TOPRIGHT", buttonUp, "TOPLEFT", -2, 0)
	
	local t = buttonDown:CreateTexture()
	t:SetAllPoints(buttonDown)
	t:SetTexture("Interface\\Buttons\\UI-MinusButton-Up")
	buttonDown:SetNormalTexture(t)
	
	t = buttonDown:CreateTexture(nil, "BACKGROUND")
	t:SetTexture("Interface\\Buttons\\UI-MinusButton-Down")
	t:SetAllPoints(buttonDown)
	buttonDown:SetPushedTexture(t)
	
	t = buttonDown:CreateTexture()
	t:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight")
	t:SetAllPoints(buttonDown)
	t:SetBlendMode("ADD")
	buttonDown:SetHighlightTexture(t)
	
	local names = {}
	for i = 1, db.maxNames do
		names[i] = frame:CreateFontString(frame:GetName().."_Name_"..i, "ARTWORK")
		names[i]:SetPoint("TOPLEFT", i == 1 and frame.rangeString or names[i-1], "BOTTOMLEFT", i == 1 and 6 or 0, -6)
		names[i]:Hide()
	end
	
	frame.closebutton = closebutton
	frame.buttonUp = buttonUp
	frame.buttonDown = buttonDown
	frame.names = names
	plugin.mainWindow = frame
	
	plugin:ApplySettings()
	
	frame:SetScript("OnUpdate", onUpdate)
	closebutton:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			self:GetParent():Hide()
		end
	end)
	buttonUp:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			plugin:IncreaseRange()
		end
	end)
	buttonDown:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			plugin:DecreaseRange()
		end
	end)
	
	return frame
end

function plugin:Reset()
	db.x = nil
	db.y = nil
	if plugin.anchor then
		plugin.anchor:ClearAllPoints()
		plugin.anchor:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -400, 400)
	end
end

function plugin:ApplySettings()
	local frame = self.mainWindow or getMainWindow()
	local rangeString = frame.rangeString
	local anchor = getAnchor()
	
	frame:SetWidth(db.Width)
	
	backdrop.bgFile = LSM:Fetch("background", db.BGTexture)
	backdrop.edgeFile = LSM:Fetch("border", db.Border)
	backdrop.edgeSize = db.BorderSize
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(unpack(db.BGColor))
	frame:SetBackdropBorderColor(unpack(db.BorderColor))
	
	rangeString:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontOutline)
	rangeString:SetTextColor(unpack(db.FontColor))
	rangeString:SetText(L.RANGE:format(db.range))
	
	for i, name in pairs(frame.names) do
		name:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontOutline)
	end
	
	frame:ClearAllPoints()
	if db.GrowUp then
		frame:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT")
	else
		frame:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT")
	end
end

function plugin:IncreaseRange()
	if db.range == 28 then return
	elseif db.range == 15 then db.range = 28
	elseif db.range == 11 then db.range = 15
	elseif db.range == 10 then db.range = 11
	end
	getMainWindow().rangeString:SetText(L.RANGE:format(db.range))
end

function plugin:DecreaseRange()
	if db.range == 10 then return
	elseif db.range == 11 then db.range = 10
	elseif db.range == 15 then db.range = 11
	elseif db.range == 28 then db.range = 15
	end
	getMainWindow().rangeString:SetText(L.RANGE:format(db.range))
end

local bandages = {21991, 34721, 38643, 34722}
function plugin:CheckRange(uID)
	local inRange = false
	if db.range == 10 then
		inRange = CheckInteractDistance(uID, 3)
	elseif db.range == 11 then
		inRange = CheckInteractDistance(uID, 2)
	elseif db.range == 15 then
		for i, bandage in pairs(bandages) do
			inRange = IsItemInRange(bandage, uID) == 1
		end
	elseif db.range == 28 then
		inRange = CheckInteractDistance(uID, 4)
	end
	
	return inRange
end

--- Show range check
-- @param show Indicates if the window should be shown or hidde (true/false)
function plugin:RW_ShowRangeCheck(message, show, range)
	if range and (range == 10 or range == 11 or range == 15 or range == 28) then
		db.range = range
	end
	if show then
		getMainWindow():Show()
	else
		getMainWindow():Hide()
	end
end

function plugin:RW_DefaultsChanged(message, option, value)
	if db[option] then db[option] = value end
	self:ApplySettings()
end

function plugin:RW_ConfigMode(message, start)
	if start then
		getAnchor():Show()
		self:RW_ShowRangeCheck("RW_ShowRangeCheck", true)
	else
		getAnchor():Hide()
		self:RW_ShowRangeCheck("RW_ShowRangeCheck", false)
	end
end

function plugin:GetOptions()	
	self.category = "Alert Plugins"
	local icon = [[Interface\Icons\INV_Misc_Map_01]]
	local function get(info)
		return db[info[#info]]
	end
	local function set(info, value)
		db[info[#info]] = value
		plugin:ApplySettings()
	end
	
	local options = {
		type = "group",
		name = "Range Check",
		icon = icon,
		get = get,
		set = set,
		args = {
			image = {
				type = "description",
				name = L.RANGE_CHECK_DESC,
				image = icon,
				width = "full",
				imageWidth = 50,
				imageHeight = 50,
				imageCoords = {0.1,0.9,0.1,0.9},
				order = 1,
			},
			enabled = {
				type = "toggle",
				name = "Enabled",
				set = function(info, value) 
					db.PluginEnabled = value
					if not db.PluginEnabled then
						self:Disable()
					else
						self:Enable()
					end
				end,
				get = function(info) return db.PluginEnabled end,
				order = 2,
			},
			headingBreak = {
				type = "header",
				name = "",
				order = 3,
			},
			GrowUp = {
				type = "toggle",
				name = L.GROW_UP,
				order = 10,
			},
			newLN1 = {
				type = "description",
				name = "",
				order = 20,
			},
			Show = {
				type = "execute",
				name = "Show",
				desc = "Show the range checker",
				func = function() plugin:SendMessage("RW_ShowRangeCheck", true) end,
				order = 21,
			},
			Hide = {
				type = "execute",
				name = "Hide",
				desc = "Hide the range checker",
				func = function() plugin:SendMessage("RW_ShowRangeCheck", false) end,
				order = 22,
			},
			newLN10 = {
				type = "description",
				name = "",
				order = 99,
			},
			Reset = {
				type = "execute",
				name = L.RESET_SETTINGS,
				desc = L.RESET_SETTINGS_DESC,
				confirm = true,
				func = function() self.db:ResetProfile() end,
				order = 100,
			},
		}
	}
	
	return options
end
